/*
 *  Copyright (C) Chaperon. All rights reserved.
 *  -------------------------------------------------------------------------
 *  This software is published under the terms of the Apache Software License
 *  version 1.1, a copy of which has been included  with this distribution in
 *  the LICENSE file.
 */

package net.sourceforge.chaperon.ant;

import org.apache.avalon.framework.logger.Logger;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;

/**
 * The Ant wrapper class for Logger.
 *
 * @author Stephan Michels
 */
public final class AntLogger implements Logger
{
  // underlying implementation to delegate to
  private final Task task;

  private int antMsgLevel = Project.MSG_ERR;

  /**
   * Create a logger that delegates to specified logger.
   *
   * @param logImpl the Ant logger to delegate to
   * @param antMsgLevel the massage level from ant
   */
  public AntLogger(Task logImpl, int antMsgLevel)
  {
    task = logImpl;
    this.antMsgLevel = antMsgLevel;
  }

  /**
   * Log a debug message.
   *
   * @param message the message
   */
  public final void debug(final String message)
  {
    if (isDebugEnabled())
      task.log(message);
  }

  /**
   * Log a debug message.
   *
   * @param message the message
   * @param throwable the throwable
   */
  public final void debug(final String message, final Throwable throwable)
  {
    if (isDebugEnabled())
      task.log(message + ":" + throwable.getMessage());
  }

  /**
   * Determine if messages of priority "debug" will be logged.
   *
   * @return true if "debug" messages will be logged
   */
  public final boolean isDebugEnabled()
  {
    return antMsgLevel == Project.MSG_DEBUG;
  }

  /**
   * Log a info message.
   *
   * @param message the message
   */
  public final void info(final String message)
  {
    if (isInfoEnabled())
      task.log(message);
  }

  /**
   * Log a info message.
   *
   * @param message the message
   * @param throwable the throwable
   */
  public final void info(final String message, final Throwable throwable)
  {
    if (isInfoEnabled())
      task.log(message + ":" + throwable.getMessage());
  }

  /**
   * Determine if messages of priority "info" will be logged.
   *
   * @return true if "info" messages will be logged
   */
  public final boolean isInfoEnabled()
  {
    return antMsgLevel >= Project.MSG_INFO;
  }

  /**
   * Log a warn message.
   *
   * @param message the message
   */
  public final void warn(final String message)
  {
    if (isWarnEnabled())
      task.log(message);
  }

  /**
   * Log a warn message.
   *
   * @param message the message
   * @param throwable the throwable
   */
  public final void warn(final String message, final Throwable throwable)
  {
    if (isWarnEnabled())
      task.log(message + ":" + throwable.getMessage());
  }

  /**
   * Determine if messages of priority "warn" will be logged.
   *
   * @return true if "warn" messages will be logged
   */
  public final boolean isWarnEnabled()
  {
    return antMsgLevel >= Project.MSG_WARN;
  }

  /**
   * Log a error message.
   *
   * @param message the message
   */
  public final void error(final String message)
  {
    if (isErrorEnabled())
      task.log(message);
  }

  /**
   * Log a error message.
   *
   * @param message the message
   * @param throwable the throwable
   */
  public final void error(final String message, final Throwable throwable)
  {
    if (isErrorEnabled())
      task.log(message + ":" + throwable.getMessage());
  }

  /**
   * Determine if messages of priority "error" will be logged.
   *
   * @return true if "error" messages will be logged
   */
  public final boolean isErrorEnabled()
  {
    return antMsgLevel >= Project.MSG_ERR;
  }

  /**
   * Log a fatalError message.
   *
   * @param message the message
   */
  public final void fatalError(final String message)
  {
    if (isFatalErrorEnabled())
      task.log(message);
  }

  /**
   * Log a fatalError message.
   *
   * @param message the message
   * @param throwable the throwable
   */
  public final void fatalError(final String message,
                               final Throwable throwable)
  {
    if (isFatalErrorEnabled())
      task.log(message + ":" + throwable.getMessage());
  }

  /**
   * Determine if messages of priority "fatalError" will be logged.
   *
   * @return true if "fatalError" messages will be logged
   */
  public final boolean isFatalErrorEnabled()
  {
    return antMsgLevel >= Project.MSG_ERR;
  }

  /**
   * Create a new child logger.
   * The name of the child logger is [current-loggers-name].[passed-in-name]
   *
   * @param name the subname of this logger
   * @return the new logger
   */
  public final Logger getChildLogger(final String name)
  {
    return this;
  }
}
